package cn.bunny.security.config;

import cn.bunny.security.custom.CustomPasswordEncoder;
import cn.bunny.security.handelr.SecurityAccessDeniedHandler;
import cn.bunny.security.handelr.SecurityAuthenticationEntryPoint;
import cn.bunny.security.handelr.SecurityLogoutSuccessHandler;
import cn.bunny.security.handelr.SecuritySessionInformationExpiredStrategy;
import cn.bunny.security.service.UserDetailsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.web.SecurityFilterChain;

@Configuration
@EnableWebSecurity
@EnableMethodSecurity
public class WebSecurityConfig {
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private CustomPasswordEncoder customPasswordEncoder;

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.authorizeHttpRequests(authorize -> {
            authorize.requestMatchers("/admin/modeler/**", "/diagram-viewer/**", "/editor-app/**", "/*.html",
                            "/admin/processImage/**", "/login/**", "/admin/system/index/login",
                            "/admin/wechat/authorize", "/admin/wechat/userInfo", "/admin/wechat/bindPhone",
                            "/favicon.ico", "/swagger-resources/**", "/webjars/**", "/v3/**", "/swagger-ui.html/**", "/doc.html").permitAll()
                    .anyRequest().authenticated();
        });

        // 注销登录
        httpSecurity
                .logout(logout -> {
                    logout.logoutSuccessHandler(new SecurityLogoutSuccessHandler());
                })
                .exceptionHandling(exception -> {
                    // 请求未授权接口
                    exception.authenticationEntryPoint(new SecurityAuthenticationEntryPoint());
                    // 没有权限访问
                    exception.accessDeniedHandler(new SecurityAccessDeniedHandler());
                })
                // 后登录的账号会使先登录的账号失效
                .sessionManagement(session -> {
                    // 最大登录数为1
                    session.maximumSessions(1).expiredSessionStrategy(new SecuritySessionInformationExpiredStrategy());
                    // 禁用session
                    // session.sessionCreationPolicy(SessionCreationPolicy.STATELESS);
                });

        httpSecurity.userDetailsService(userDetailsService).passwordManagement(customPasswordEncoder);

        return httpSecurity.build();
    }
}
